(************************************************************************)
(*                                                                      *)
(*                    Version 2.1a with error checks                    *)
(*                                                                      *)
(************************************************************************)


MODULE SETsTest;

(************************************************************************)
(*                                                                      *)
(*                          S E T s T e s t                             *)
(*                                                                      *)
(************************************************************************)
(*                                                                      *)
(* Distribution:                                                        *)
(*                                                                      *)
(*   I dedicate this  module  to the public domain.  Please do not send *)
(*   any money.  I do not take any warranty for the correctness of this *)
(*   software.  Especially I cannot  be held responsible for any damage *)
(*   that occurs because of the use of this module.                     *)
(*                                                                      *)
(*   Do not remove my name/address from one of the headers.		*)
(*                                                                      *)
(*   Please send all  critics and suggestions to me.  I will try to use *)
(*   them in future releases.                                         	*)
(*                                                                      *)
(************************************************************************)
(*                                                                      *)
(* Purpose:                                                             *)
(*                                                                      *)
(*   SETsTest just tests the module  SETs  for integrity.  You can find *)
(*   a file 'Results.txt'  in your distribution which contains the out- *)
(*   put of a correct  running test.  Just compare  this with  your own *)
(*   results and report differences to the author.                      *)
(*                                                                      *)
(************************************************************************)
(*                                                                      *)
(* Revisions:                                                           *)
(*                                                                      *)
(*   01/27/92	Version 2.1						*)
(*   01/27/92	New TestSETRangeExtractAnyElement [BR]			*)
(*   11/04/91	New TestAutoCompact, Version 2.0 [BR]			*)
(*   08/29/91	Test of SETGet/ReplaceValue [BR]			*)
(*   06/13/91   Versions 1.0a,b,c                                       *)
(*                                                                      *)
(************************************************************************)

  (* Import from all standard modules. Thus they will also be compiled when
     a make is performed *)

  (* the class SETofTESTclass is created in SETofTEST with a sorting order
     on the two key components KEY1 and KEY2 *)
  FROM SETofTEST IMPORT
    SETofTESTclass;

  FROM SETofCARDINAL IMPORT
    SETofCARDINALclass;

  FROM SETofSHORTCARD IMPORT
    SETofSHORTCARDclass;

  FROM SETofSHORTINT IMPORT
    SETofSHORTINTclass;

  FROM SETofINTEGER IMPORT
    SETofINTEGERclass;

  FROM SETofREAL IMPORT
    SETofREALclass;

  FROM SETofWORD IMPORT
    SETofWORDclass;

  FROM SETofCHAR IMPORT
    SETofCHARclass;

  FROM SETs IMPORT
    SETType,
    SETIsDisjunct,
    SETIsSubset,
    SETCreate,
    SETGotoElement,
    SETIsStrictSubset,
    SETGotoMinimum,
    SETGotoMaximum,
    SETInsert,
    SETCard,
    SETCompareResultType,
    SETCompareProcType,
    SETSetUndefined,
    SETCheckConsistency,
    SETCardUncompacted,
    SETIsEmpty,
    SETCreateForN,
    SETInsertUptoN,
    SETCopy,
    SETGetValue,
    SETReplaceValue,
    SETGet,
    SETLoop,
    SETRangeLoop,
    SETRangeExtractAnyElement,
    SETKill,
    SETUnion,
    SETSymmetricDifference,
    SETDifference,
    SETIntersection,
    SETCompact,
    SETCompare,
    SETIn,
    SETDeleteElement,
    SETRemoveElement,
    SETExtractAnyElement;

  FROM SYSTEM IMPORT
    TSIZE;

  FROM TESTdef IMPORT
    TEST;

  FROM SimpleIO IMPORT
    WriteString,
    WriteLn,
    WriteChar,
    WriteCard;
  
  

  VAR
    i: CARDINAL;
    t: TEST;
    s1,s2: SETType;

  PROCEDURE Show(P: ARRAY OF CHAR; S: SETType);

  VAR
    i: CARDINAL;
    val: TEST;
    Found: BOOLEAN;

  BEGIN
    SETCheckConsistency(S);
    WriteString("begin "); WriteString(P); WriteLn;
    WriteString("  log#="); WriteCard(SETCard(S),2);
    WriteString(" phys#="); WriteCard(SETCardUncompacted(S),2);
    WriteLn;
    SETLoop(S);
    i := 0;
    REPEAT
      SETGet(S,val,Found);
      IF (Found) THEN 
        INC(i);
        WriteCard(i,4);
        WriteString(". (");
        WriteCard(CARDINAL(val.KEY1),2);
        WriteString(",");
        WriteCard(CARDINAL(val.KEY2),2);
        WriteString(',"');
        WriteString(val.CTS);
        WriteString('")');
        WriteLn
      END
    UNTIL (NOT Found);
    IF (SETIsEmpty(S)) THEN WriteString("  empty"); WriteLn END;
    WriteString("end "); WriteString(P);
    WriteLn
  END Show;

  PROCEDURE Enter(P: ARRAY OF CHAR);

  BEGIN
    WriteLn;
    WriteString("ENTER ");
    WriteString(P);
    WriteLn;
  END Enter;
  
  PROCEDURE Leave;
  
  BEGIN
    WriteString("EXIT"); 
    WriteLn
  END Leave;

  (* Key1Cmp defines a partial order on the set elements just for the first
     key component KEY1 *)
  PROCEDURE Key1Cmp(VAR a,b: TEST): SETCompareResultType;

  BEGIN
    IF (a.KEY1<b.KEY1) THEN RETURN(SETLess) END;
    IF (a.KEY1>b.KEY1) THEN RETURN(SETGreater) END;
    RETURN(SETEqual)
  END Key1Cmp;


  PROCEDURE RangeShow(K: CARDINAL; S: SETType);

  VAR
    i: CARDINAL;
    val: TEST;
    Found: BOOLEAN;

  BEGIN
    SETCheckConsistency(S);
    WriteString("begin range loop for KEY1="); 
    WriteCard(K,1); WriteLn;
    WriteString("  log#="); WriteCard(SETCard(S),2);
    WriteString(" phys#="); WriteCard(SETCardUncompacted(S),2);
    WriteLn;
    (* do not forget to specify a complete set element. KEY2 and CTS are
       however not used in Key1Cmp. So their initialization may be omitted *)
    t.KEY1 := K;
    t.KEY2 := 0;
    t.CTS[0] := 0C;
    SETRangeLoop(S,SETCompareProcType(Key1Cmp),t);
    i := 0;
    REPEAT
      SETGet(S,val,Found);
      IF (Found) THEN 
        INC(i);
        WriteCard(i,4);
        WriteString(". (");
        WriteCard(CARDINAL(val.KEY1),2);
        WriteString(",");
        WriteCard(CARDINAL(val.KEY2),2);
        WriteString(',"');
        WriteString(val.CTS);
        WriteString('")');
        WriteLn
      END
    UNTIL (NOT Found);
    IF (SETIsEmpty(S)) THEN WriteString("  empty"); WriteLn END;
    WriteString("end range loop");
    WriteLn; WriteLn
  END RangeShow;

  PROCEDURE TestSETRemoveElement;

  VAR
    i: CARDINAL;
    s: SETType;

  BEGIN
    Enter("TestSETRemoveElement");
    SETCopy(s1,s);
    FOR i:=10 TO 20 DO
      t.KEY1 := i;
      t.KEY2 := i;
      SETRemoveElement(s,t)
    END;
    Show("first set after removing 10..20",s);
    SETKill(s);
    Leave;
  END TestSETRemoveElement;

  PROCEDURE TestSETDeleteElement;
  
  VAR
    s: SETType;
    i: CARDINAL;
    f: BOOLEAN;
        
  BEGIN
    Enter("TestSETDeleteElement");
    SETCopy(s1,s);
    FOR i:=10 TO 30 DO
      t.KEY1 := i;
      t.KEY2 := i;
      SETDeleteElement(s,t,f)
    END;
    Show("first set after deleting 10..30",s);
    SETKill(s);
    Leave;
  END TestSETDeleteElement;

  PROCEDURE TestSETExtractAnyElement;
  
  VAR
    i: CARDINAL;
    s: SETType;
    f: BOOLEAN;
    
  BEGIN
    Enter("TestSETExtractAnyElement");
    SETCopy(s1,s);
    FOR i:=1 TO 10 DO
      SETExtractAnyElement(s,t,f)
    END;
    Show("first set after extracting 10 elements",s);
    FOR i:=1 TO 20 DO
      SETExtractAnyElement(s,t,f)
    END;
    Show("first set after extracting 20 more elements",s);
    SETKill(s);
    Leave
  END TestSETExtractAnyElement;

  PROCEDURE TestSETCompact;

  VAR
    i: CARDINAL;
    s: SETType;
    f: BOOLEAN;
    
  BEGIN
    Enter("TestSETCompact");
    SETCopy(s1,s);
    FOR i:=1 TO 10 DO
      SETExtractAnyElement(s,t,f)
    END;
    SETCompact(s);
    Show("first set after extracting 10 elements and compacting",s);
    SETKill(s);
    Leave
  END TestSETCompact;

  PROCEDURE TestSETIn;

  VAR
    i: CARDINAL;
    s: SETType;
    
  BEGIN
    Enter("TestSETIn");
    SETCopy(s1,s);
    FOR i:=1 TO 20 DO
      t.KEY1 := i;
      t.KEY2 := i;
      IF (NOT SETIn(s,t)) THEN
        WriteString("### SETIn Error (1)"); 
        WriteLn
      END
    END;
    FOR i:=21 TO 30 DO
      t.KEY1 := i;
      t.KEY2 := i;
      IF (SETIn(s,t)) THEN
        WriteString("### SETIn Error (2)"); 
        WriteLn
      END
    END;
    SETKill(s);
    Leave
  END TestSETIn;

  PROCEDURE TestSETCompare;

  VAR
    i: CARDINAL;
    r,s: SETType;
    eq,le,gr: BOOLEAN;
    
  BEGIN
    Enter("TestSETCompare");
    SETCopy(s1,r);
    SETCopy(s2,s);
    SETCompare(r,s,eq,le,gr);
    IF (eq) OR (le) OR (gr) THEN
      WriteString("### SETCompare Error (1)");
      WriteLn
    END;
    FOR i:=21 TO 25 DO
      t.KEY1 := i;
      t.KEY2 := i;
      SETRemoveElement(s,t);
    END;
    SETCompare(r,s,eq,le,gr);
    IF (eq) OR (le) OR (NOT gr) THEN
      WriteString("### SETCompare Error (2)");
      WriteLn
    END;
    FOR i:=1 TO 15 DO
      t.KEY1 := i;
      t.KEY2 := i;
      SETRemoveElement(r,t);
    END;
    SETCompare(r,s,eq,le,gr);
    IF (eq) OR (NOT le) OR (gr) THEN
      WriteString("### SETCompare Error (3)");
      WriteLn
    END;
    SETKill(r);
    SETKill(s);
    Leave
  END TestSETCompare;

  PROCEDURE TestSETGetValue;

  VAR
    i: CARDINAL;
    r: SETType;
    f: BOOLEAN;
    
  BEGIN
    Enter("TestSETGetValue");
    SETCopy(s1,r);
    FOR i:=15 TO 20 DO
      t.KEY1 := i;
      t.KEY2 := i;
      t.CTS[0] := "Z";
      t.CTS[1] := 0C;
      SETGetValue(r,t,f);
      IF (NOT f) OR (t.KEY1#i) OR (t.KEY2#i) 
      OR (ORD(t.CTS[0])#ORD('A')+i-1) THEN
        WriteString("### SETGetValue error");
        WriteLn
      END
    END;
    SETKill(r);
    Leave
  END TestSETGetValue;

  PROCEDURE TestSETReplaceValue;

  VAR
    i: CARDINAL;
    r: SETType;
    f: BOOLEAN;
    
  BEGIN
    Enter("TestSETReplaceValue");
    SETCopy(s1,r);
    FOR i:=15 TO 20 DO
      t.KEY1 := i;
      t.KEY2 := i;
      t.CTS[0] := "Z";
      t.CTS[1] := 0C;
      SETReplaceValue(r,t,f);
      IF (NOT f) THEN
        WriteString("### SETReplaceValue error");
        WriteLn
      END
    END;
    Show("After replacing some elements",r);
    SETKill(r);
    Leave
  END TestSETReplaceValue;

  PROCEDURE TestSETIsSubset;

  VAR
    i: CARDINAL;
    r,s: SETType;
    
  BEGIN
    Enter("TestSETIsSubset");
    SETCopy(s1,r);
    SETCopy(s2,s);
    IF (SETIsSubset(r,s)) OR (SETIsSubset(s,r)) THEN
      WriteString("### SETIsSubset Error (1)");
      WriteLn
    END;
    FOR i:=21 TO 25 DO
      t.KEY1 := i;
      t.KEY2 := i;
      SETRemoveElement(s,t);
    END;
    IF (SETIsSubset(r,s)) OR (NOT SETIsSubset(s,r)) THEN
      WriteString("### SETIsSubset Error (2)");
      WriteLn
    END;
    FOR i:=1 TO 15 DO
      t.KEY1 := i;
      t.KEY2 := i;
      SETRemoveElement(r,t);
    END;
    IF (NOT SETIsSubset(r,s)) OR (SETIsSubset(s,r)) THEN
      WriteString("### SETIsSubset Error (3)");
      WriteLn
    END;
    SETKill(r);
    SETKill(s);
    Leave
  END TestSETIsSubset;

  PROCEDURE TestSETIsStrictSubset;

  VAR
    i: CARDINAL;
    r,s: SETType;
    
  BEGIN
    Enter("TestSETIsStrictSubset");
    SETCopy(s1,r);
    SETCopy(s2,s);
    IF (SETIsStrictSubset(r,s)) OR (SETIsStrictSubset(s,r)) THEN
      WriteString("### SETIsStrictSubset Error (1)");
      WriteLn
    END;
    FOR i:=21 TO 25 DO
      t.KEY1 := i;
      t.KEY2 := i;
      SETRemoveElement(s,t);
    END;
    IF (SETIsStrictSubset(r,s)) OR (NOT SETIsStrictSubset(s,r)) THEN
      WriteString("### SETIsStrictSubset Error (2)");
      WriteLn
    END;
    FOR i:=1 TO 15 DO
      t.KEY1 := i;
      t.KEY2 := i;
      SETRemoveElement(r,t);
    END;
    IF (NOT SETIsStrictSubset(r,s)) OR (SETIsStrictSubset(s,r)) THEN
      WriteString("### SETIsStrictSubset Error (3)");
      WriteLn
    END;
    SETKill(r);
    SETKill(s);
    Leave
  END TestSETIsStrictSubset;

  PROCEDURE TestSETIsDisjunct;

  VAR
    i: CARDINAL;
    r,s: SETType;
    
  BEGIN
    Enter("TestSETIsDisjunct");
    SETCopy(s1,r);
    SETCopy(s2,s);
    IF (SETIsDisjunct(r,s)) THEN
      WriteString("### SETIsDisjunct Error (1)");
      WriteLn
    END;
    FOR i:=10 TO 20 DO
      t.KEY1 := i;
      t.KEY2 := i;
      SETRemoveElement(s,t);
    END;
    IF (NOT SETIsDisjunct(s,r)) THEN
      WriteString("### SETIsDisjunct Error (2)");
      WriteLn
    END;
    SETKill(r);
    SETKill(s);
    Leave
  END TestSETIsDisjunct;

  PROCEDURE TestSETUnion;
  
  VAR
    r,s: SETType;
    
  BEGIN
    Enter("TestSETUnion");
    SETCopy(s1,r);
    SETCopy(s2,s);
    SETUnion(r,s);
    Show("Union of the first and second set",r);
    SETKill(r);
    SETKill(s);
    Leave
  END TestSETUnion;

  PROCEDURE TestSETIntersection;
  
  VAR
    r,s: SETType;
    
  BEGIN
    Enter("TestSETIntersection");
    SETCopy(s1,r);
    SETCopy(s2,s);
    SETIntersection(r,s);
    Show("Intersection of the first and second set",r);
    SETKill(r);
    SETKill(s);
    Leave
  END TestSETIntersection;

  PROCEDURE TestSETDifference;
  
  VAR
    r,s: SETType;
    
  BEGIN
    Enter("TestSETDifference");
    SETCopy(s1,r);
    SETCopy(s2,s);
    SETDifference(r,s);
    Show("Difference first - second set",r);
    SETKill(r);
    SETKill(s);

    SETCopy(s1,r);
    SETCopy(s2,s);
    SETDifference(s,r);
    Show("Difference second -first set",s);
    SETKill(r);
    SETKill(s);
    Leave
  END TestSETDifference;

  PROCEDURE TestSETSymmetricDifference;
  
  VAR
    r,s: SETType;
    
  BEGIN
    Enter("TestSETSymmetricDifference");
    SETCopy(s1,r);
    SETCopy(s2,s);
    SETSymmetricDifference(r,s);
    Show("Symmetric difference of the first and second set",r);
    SETKill(r);
    SETKill(s);
    Leave
  END TestSETSymmetricDifference;

  PROCEDURE TestGotoProcs;

  VAR
    s: SETType;
    f: BOOLEAN;
    
  BEGIN
    Enter("TestGotoProcs");
    SETCopy(s1,s);
    t.KEY1 := 10;
    t.KEY2 := 10;

    SETGotoElement(s,t,f);
    SETGet(s,t,f);
    IF (NOT f) OR (t.KEY1#10) OR (t.KEY2#10) 
    OR (t.CTS[0]#CHR(ORD('A')+10-1)) THEN
      WriteString("### SETGotoElement error");
      WriteLn
    END;
    
    SETGotoMinimum(s);
    SETGet(s,t,f);
    IF (NOT f) OR (t.KEY1#1) OR (t.KEY2#1) OR (t.CTS[0]#'A') THEN
      WriteString("### SETGotoMinimum error");
      WriteLn
    END;
    
    SETGotoMaximum(s);
    SETGet(s,t,f);
    IF (NOT f) OR (t.KEY1#20) OR (t.KEY2#20) 
    OR (t.CTS[0]#CHR(ORD('A')+20-1)) THEN
      WriteString("### SETGotoMaximum error");
      WriteLn
    END;
    
    SETKill(s);
    Leave
  END TestGotoProcs;

  PROCEDURE TestSETRangeLoop;

  VAR
    i: CARDINAL;
    s: SETType;
  
  BEGIN
    Enter("TestSETRangeLoop");
    (* creation of set which contains several elements which share the same
       first component KEY1 *)
    SETCreate(s,SETofTESTclass);
    FOR i:=1 TO 20 DO
      (* i even --> KEY1 = 1, else 2 *)
      t.KEY2 := i;
      t.CTS[0] := CHR(ORD('A')+i-1);
      t.CTS[1] := 0C;
      IF (i MOD 2=0) THEN
        t.KEY1 := 1
      ELSE
        t.KEY1 := 2
      END;
      SETInsert(s,t);
    END;

    (* Show those elements whose KEY1 is 2 *)
    RangeShow(2,s);

    (* Show those elements whose KEY1 is 1 *)
    RangeShow(1,s);
    
    SETKill(s);
    Leave
  END TestSETRangeLoop;

  PROCEDURE TestSETRangeExtractAnyElement;

  VAR
    i: CARDINAL;
    s: SETType;
    x: TEST;
    f: BOOLEAN;
  
  BEGIN
    Enter("TestSETRangeExtractAnyElement");
    (* creation of set which contains several elements which share the same
       first component KEY1 *)
    SETCreate(s,SETofTESTclass);
    FOR i:=1 TO 20 DO
      (* i even --> KEY1 = 1, else 2 *)
      t.KEY2 := i;
      t.CTS[0] := CHR(ORD('A')+i-1);
      t.CTS[1] := 0C;
      IF (i MOD 2=0) THEN
        t.KEY1 := 1
      ELSE
        t.KEY1 := 2
      END;
      SETInsert(s,t);
    END;

    (* Loop over and deleting of those elements whose KEY1 is 2 *)
    t.KEY1 := 1;
    REPEAT
      SETRangeExtractAnyElement(s,SETCompareProcType(Key1Cmp),t,x,f)
    UNTIL (NOT f);
    Show("Set contains only KEY1=2 elements",s);
    SETKill(s);
    Leave
  END TestSETRangeExtractAnyElement;

  PROCEDURE TestAutoCompact;

  VAR
    i: CARDINAL;
    s: SETType;
    f: BOOLEAN;
  
  BEGIN
    Enter("TestAutoCompact");
    (* sorted creation of a set with 200 elements. This is a test. YOU SHOULD
       NORMALLY USE SETCreateForN and SETInsertUptoN TO DO THIS!!! Internally
       there will be lots of compactifications to avoid a badly balanced
       set tree *)
    SETCreate(s,SETofTESTclass);
    FOR i:=1 TO 200 DO
      t.KEY1 := i;
      t.KEY2 := i;
      t.CTS[0] := 'A';
      t.CTS[1] := 0C;
      SETInsert(s,t);
    END;

    (* Deletion of 180 elements. Internally there will be compactifications
       to avoid a bad storage usage. Note that SETExtractAnyElement does not
       automatically compact. Therefore SETRemoveElement is used here *)
    FOR i:=1 TO 180 DO
      t.KEY1 := i;
      t.KEY2 := i;
      t.CTS[0] := 'A';
      t.CTS[1] := 0C;
      SETRemoveElement(s,t)
    END;

    Show("Still 20 elements there",s);

    SETKill(s);
    Leave
  END TestAutoCompact;

BEGIN
  (* not necessary (just an example) *)
  SETSetUndefined(s1);
  SETSetUndefined(s2);

  (* create test sets *)
  SETCreate(s1,SETofTESTclass);
  FOR i:=1 TO 20 DO 
    t.KEY1 := i;
    t.KEY2 := i;
    t.CTS[0]  := CHR(ORD('A')+i-1);
    t.CTS[1]  := CHR(ORD('A')+i-1);
    t.CTS[2]  := CHR(ORD('A')+i-1);
    t.CTS[3]  := 0C;
    SETInsert(s1,t) 
  END;
  Show("first set",s1);

  SETCreate(s2,SETofTESTclass);
  FOR i:=10 TO 25 DO 
    t.KEY1 := i;
    t.KEY2 := i;
    t.CTS[0]  := CHR(ORD('A')+i-1);
    t.CTS[1]  := CHR(ORD('A')+i-1);
    t.CTS[2]  := CHR(ORD('A')+i-1);
    t.CTS[3]  := 0C;
    SETInsert(s2,t) 
  END;
  Show("second set",s2);

  TestSETRemoveElement;
  TestSETDeleteElement;
  TestSETExtractAnyElement;
  TestSETCompact;
  TestSETIn;
  TestSETCompare;
  TestSETIsSubset;
  TestSETIsStrictSubset;
  TestSETIsDisjunct;
  TestSETGetValue;
  TestSETReplaceValue;
  TestSETUnion;
  TestSETIntersection;
  TestSETDifference;
  TestSETSymmetricDifference;
  TestSETRangeLoop;
  TestSETRangeExtractAnyElement;

  TestGotoProcs;
  TestAutoCompact;
  
  SETKill(s2);
  SETKill(s1);

  WriteLn;
  (* test the module Error by creating a SETs runtime error *)
  WriteString("Testing Error");
  WriteLn;
  SETInsert(s1,t)

END SETsTest.
