(************************************************************************)
(*                                                                      *)
(*                    Version 2.1a with error checks                    *)
(*                                                                      *)
(************************************************************************)


MODULE Example;

(************************************************************************)
(*                                                                      *)
(*                            E x a m p l e				*)
(*                                                                      *)
(************************************************************************)
(*                                                                      *)
(* Revisions:                                                           *)
(*                                                                      *)
(*   01/27/92	First version for SETs 2.1 [BR]				*)
(*                                                                      *)
(************************************************************************)

(* SETofCARDINAL exports the variable SETofCARDINALclass which may be used
   to create sets of CARDINAL numbers *)
FROM SETofCARDINAL IMPORT
  SETofCARDINALclass;

(* SETs exports the generic opaque data type SETType and all operations *)
FROM SETs IMPORT
  SETCreate,
  SETKill,
  SETInsert,
  SETCopy,
  SETIntersection,
  SETUnion,
  SETDifference,
  SETSymmetricDifference,
  SETLoop,
  SETGet,
  SETType;

FROM InOut IMPORT
  WriteString,
  WriteCard,
  WriteLn;

VAR
  set1,set2,tmpset: SETType;
  i,j: CARDINAL;

  PROCEDURE ShowSet(    prompt: ARRAY OF CHAR;
  		        s     : SETType);

  VAR
    i    : CARDINAL;
    Found: BOOLEAN;
  
  BEGIN
    (* output the prompt *)
    WriteString(prompt);
    WriteString(": ");

    (* Start a loop over set s *)
    SETLoop(s);

    (* Get and output the elements *)
    REPEAT
      SETGet(s,i,Found);
      IF (Found) THEN
        WriteCard(i,0);
	WriteString(" ")
      END
    UNTIL (NOT Found);
    WriteLn
  END ShowSet;

BEGIN
  (* Creation of two CARDINAL sets *)
  SETCreate(set1,SETofCARDINALclass);
  SETCreate(set2,SETofCARDINALclass);

  (* Insert 0..100 into the first set. Programs with efficiency requirements
     should use SETCreateForN/SETInsertUptoN, but its just an example... *)
  FOR i:=0 TO 100 DO
    SETInsert(set1,i)
  END;
  ShowSet("First set      ",set1);

  (* Insert 0..9, 20..29, 40..49, 60..69, 80..89, 100..109 into set2. This
     should also be made the other way for efficiency reasons *)
  FOR j:=0 TO 5 DO
    FOR i:=20*j TO 20*j+9 DO
      SETInsert(set2,i)
    END
  END;
  ShowSet("Second set     ",set2);

  (* Calculate intersection, union, two differences and a symmetric
     difference. Therefore a copy of the latter target set will be created
     before each operation *)

  (* --- Intersection --- *)
  SETCopy(set1,tmpset);
  SETIntersection(tmpset,set2);
  ShowSet("Intersection   ",tmpset);
  SETKill(tmpset);

  (* --- Union --- *)
  SETCopy(set1,tmpset);
  SETUnion(tmpset,set2);
  ShowSet("Union          ",tmpset);
  SETKill(tmpset);

  (* --- Difference 1-2 --- *)
  SETCopy(set1,tmpset);
  SETDifference(tmpset,set2);
  ShowSet("Difference 1-2 ",tmpset);
  SETKill(tmpset);

  (* --- Difference 2-1 --- *)
  SETCopy(set2,tmpset);
  SETDifference(tmpset,set1);
  ShowSet("Difference 2-1 ",tmpset);
  SETKill(tmpset);

  (* --- symm. Difference --- *)
  SETCopy(set1,tmpset);
  SETSymmetricDifference(tmpset,set2);
  ShowSet("symm.Difference",tmpset);
  SETKill(tmpset);

  (* Kill the sets *)
  SETKill(set1);
  SETKill(set2)
END Example.
